@@ -156,6 +156,8 @@ def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggre
   elif (type_ == 'futures_klines') and (symbol is not None) and (aggreg is not None):
     startTime = int(start_time.timestamp()*1000)
     endTime = int(end_time.timestamp()*1000)
+    previousStartTime = startTime
+
     while startTime<endTime:
       time_col = 'time'
       try:
@@ -165,7 +167,11 @@ def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggre
         output = pd.concat([output, pd.DataFrame(client.futures_klines(symbol = symbol, interval = aggreg, startTime = startTime, endTime = endTime, limit = 1000))])
       except json.decoder.JSONDecodeError:
         print('Could not convert to Json. Passing @time:', startTime)
-      startTime = output.loc[:,0].max()
+      previousStartTime = startTime
+      startTime = output.loc[:,'time'].max()
+      if previousStartTime == startTime :
+          break
+
     output.columns = ['time', 'open', 'high', 'low', 'close', 'volume', 'close_time', 'quote_asset_vol', 'ntrades', 'takerbuy_base_asset_vol', 'takerbuy_quote_asset_vol', 'ignore']
   elif type_ == 'wallet_history':
     startTime = int(start_time.timestamp()*1000)
@@ -188,6 +194,8 @@ def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggre
   elif (type_ == 'spot_klines')  and (symbol is not None) and (aggreg is not None):
     startTime = int(start_time.timestamp()*1000)
     endTime = int(end_time.timestamp()*1000)
+    previousStartTime = startTime
+
     while startTime<endTime:
       time_col = 'time'
       try:
@@ -196,7 +204,11 @@ def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggre
         output = pd.concat([output, pd.DataFrame(client.get_historical_klines(symbol = symbol, interval = aggreg, start_str = startTime, end_str = endTime, limit = 500))])
       except json.decoder.JSONDecodeError:
         print('Could not convert to Json. Passing @time:', startTime)
-      startTime = output.loc[:,0].max()
+      previousStartTime = startTime
+      startTime = output.loc[:,'time'].max()
+      if previousStartTime == startTime :
+          break
+
     output.columns = ['time', 'open', 'high', 'low', 'close', 'volume', 'close_time', 'quote_asset_vol', 'ntrades', 'takerbuy_base_asset_vol', 'takerbuy_quote_asset_vol', 'ignore']
   else:
     print('Wrong param combination.')
@@ -656,7 +668,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
 
     x = td[['traded_eth', 'avg_eth_price', 'amount_traded_usd_eth', 'sens_trade_eth', 'open_eth_cc', 'open_eth_bnc',
             'traded_xbt', 'avg_btc_price', 'amount_traded_usd_btc', 'sens_trade_xbt', 'open_btc_cc', 'open_btc_bnc',
-            'valuation', 'pnl',
+            'valuation', 'transfers_usd',
             'trading_fees_eth_usd', 'funding_fees_eth_usd', 'slippage_eth_price_cc', 'slippage_eth_cash_cc','slippage_eth_price_bnc', 'slippage_eth_cash_bnc',
             'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_cc', 'slippage_btc_cash_cc','slippage_btc_price_bnc', 'slippage_btc_cash_bnc' ]]
 
@@ -665,7 +677,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
                  'open_eth_cc', 'open_eth_exchange',
                  'quantity_traded_btc', 'avg_traded_price_btc', 'amount_traded_usd_btc', 'n_trades_btc',
                  'open_btc_cc', 'open_btc_exchange',
-                 'wallet_amount_usd', 'pnl',
+                 'wallet_amount_usd', 'transfers_usd',
                  'trading_fees_usd_eth', 'funding_fees_usd_eth', 'slippage_eth_price_cc', 'slippage_eth_cash_cc','slippage_eth_price_bnc', 'slippage_eth_cash_bnc',
                  'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_cc', 'slippage_btc_cash_cc','slippage_btc_price_bnc', 'slippage_btc_cash_bnc' ]
 
